function [fval,sarcoefnew,macoefnew,vnew,fvalnew,xxnew] = ...
    estimateSARMA(data,p,q,maxiter,spectrum,sarcoefi,macoefi,vi,useunc)

%input: data(t,:), t=1,2,3,...,T, data set
%
%          p - coefficients in autoregressive part
%          q - number of lags in matrix moving average part
%
%  if user supplies sarcoefi, macoefi, vi, then these are used for initial
%  conditions for estimation, otherwise an initial, consistent estimator
%  will be used.
%  if spectrum = 1, use the frequency domain approximation to the
%  likelihood, if = 0, use the exact gaussian density
%  You must have either 5, 8 or 9 input arguments. With 5, the routine uses
%  rob's initial consistent estimator of the sarma. With 8, the supplied
%  initial guesses are used. Alternatively, if sarcoefi = [], then inputs
%  are expected to be supplied manually in the code (this is useful for
%  debugging).
%  useunc need not be supplied, but if it is then = 0 means the
%  optimization prints out intermediate information and =1 means none is
%  printed.
%
%output: 
%
%    1 by p vector, sarcoef; macoef = [macoef(1),...,macoef(q)], where macoef(i) is a matrix 
%
%    y_(t) =   sarcoef(1)*y_(t-1) + ... + sarcoef(p)*y_(t-n) + u_(t) + marcoef(1)*u_(t-1) + ... + marcoef(q)* u(t-q)
%   (the data are assumed to have been demeaned)
%
%    erz - fitted disturbances, u_(t)
%    V  variance-covariance matrix of erz 
%
%   fval - minus, log likelihood of initial guess at parameters
%

xx = [];

[T,N]=size(data);
if N > T
    error('fatal (estimateSARMA) more variables than dates, data vector should probably be transposed')
end

if nargin == 5
    
    %get an initial, consistent, estimate of the SARMA:
    [sarcoef,macoef,v] = stepone(data,p,q);
    
elseif nargin >= 6
    
    if isempty(sarcoefi)
        
        %unrestricted estimation with cev data and spectral approximation
        xx=[ 1.96037125054776
            -0.96324275481070
            -1.93214840269947
            -0.01017293423027
            0.02884860307693
            -0.97733429548642
            0.93160824803020
            -0.00091221035973
            -0.02779075454720
            -0.00101501718267
            0.00004097194655
            -0.00000996275371
            0.00006767991963];
        
        %unrestricted estimation with cev data and exact gaussian density
        xx=[ 1.74485761875045
            -0.76790909850603
            -1.86558333508802
            0.16358890498323
            -0.19236757852091
            -0.23384410395627
            0.89684606507760
            0.15092373566022
            0.18905083210265
            -0.13116780951966
            0.00003750415767
            -0.00000506702035
            0.00003931185551];
        
    else    
        
        sarcoef=sarcoefi;
        macoef=macoefi;
        v=vi;
        
    end
    
elseif nargin ~= 5 & nargin ~= 8 & nargin ~= 9
    
    error('fatal (estimateSARMA) wrong number of input arguments')
    
end

if spectrum == 1    
    [per] = getspectrum(data);    
else
    per=[];
end

if nargin < 9;
    useunc =0;
end

if isempty(xx);
    [xx] = coef2opt(sarcoef,macoef,v);
end

[fval] = SARMA(xx,p,q,N,data,per);

if maxiter == 0
    
    fvalnew=fval;
    xxnew=xx;
    sarcoefnew=[];
    macoefnew=[];
    vnew=[];
    fvalnew=[];
    
elseif maxiter > 0    
    
    if useunc == 0
        
        optionals = optimset('LevenbergMarquardt','on','Display','iter','TolFun',1e-8,'TolX',1e-8,'MaxFun',maxiter,'Diagnostics','on','MaxFunEvals',maxiter,'MaxIter',maxiter);
        [xxnew,fvalnew,exitflag] = fminsearch(@SARMA,xx,optionals,p,q,N,data,per)
        
    else
        
        optionals = optimset('Display','iter','TolFun',1e-8,'TolX',1e-8,'Diagnostics','on');
        [xxnew,fvalnew,exitflag] = fminunc(@SARMA,xx,optionals ,p,q,N,data,per)
        
    end
    
    if exitflag == 1
        disp('(estimateSARMA) Optimization converged succesfully')
    elseif exitflag == 0 
        disp('(estimateSARMA) Iteration limit reached')
    end
    
    if isnan(fvalnew) == 1 | abs(imag(fvalnew)) > .1e-10
        error('(estimateSARMA) fatal function value not admissible')
    end
    
    [sarcoefnew,macoefnew,vnew] = opt2coef(xxnew,p,q,N);
    
end