function [rhol,sigl,sigz,rhox,sigx,R,se,H,fval,xxnew,Yout] =  ...
    estimatemodel(Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,sigx, ...
    RR,rhol,sigl,sigz,option,sht,const,stderr,measerr,maxiter,spectrum)

% this estimates the two-shock rbc model
% inputs: 
%    Y(:,t), t=1,...,T, the data used in the analysis
%   R initial estimate of the measurement error on the data
%   rrhol, ssigl, ssigz initial estimate of autocorrelation of shock on
%   preferences, std of preference shock, std of tech shock
%  option = 1 if cointegrating vectors imposed, =2 if ckm strategy followed
%  sht = 1 if only output growth and labor used in analysis, otherwise all
%  four used
%  const=0 if no constant term is estimated (this is the only case  it can
%  handle now)
%  stderr = 1 if standard errors are desired
%  measerr: if = 1, and sht = 0, then include measurement error in
%  estimation; if = 0, then just use R, but do not estimate
%  maxiter maximum permitted number of iterations
%  if spectrum = 1, then use spectral approximation to gaussian density,
%  otherwise use exact density
%
% output
% rhol, sigl, sigz: estimate of parameters
% se standard errors (if se =[] and stderr = 1, then there was a problem
% computing the standard errors...probably badly behaved hessian)
% Yout, If const=0, then the mean is removed from the data and put in Yout.
% Otherwise, Yout just contains the same data that was read in at the start.
% 

[m,T]=size(Y); 
if m > T
    error('fatal (estimatemodel) number of variables exceeds number of observations....or, Y should be transposed')
end

if const == 0%if no constant is to be used in the analysis, then take mean out of the data
    mm=mean(Y');
    for ii = 1:m
        Y(ii,:)=Y(ii,:)-ones(1,T)*mm(ii);
    end
end

Yout=Y;

xx=zeros(7,1);
if sht >= 1 | measerr == 0
    xx=zeros(3,1);
    RR=zeros(2,2);
    if sht == 3;RR=zeros(3,3);end
end

if sht == 0 & measerr == 0
    RR=.0001*eye(4);
end

xx(3)=-log(1/rhol-1);
xx(2)=log(sigl);
xx(1)=log(sigz);

if sht == 3
    xx(4)=-log(1/rhox-1);
    xx(5)=log(sigx);
end

if sht == 0 & measerr == 1
    for ii = 1:4
        xx(3+ii)=sqrt(RR(ii,ii));
    end
end

if spectrum == 1    
    [per] = getspectrum(Y);    
else
    per=[];
end

optype = 1;%=1, MATLAB's minimization; =2, simulated annealing;  =4, Sims' optimization routine
restr=1;
rrhol=rhol;
ssigl=sigl;
ssigx=sigx;
rrhox=rhox;

[floglik,H,F,W,R,init_V] = ckm(xx,Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rrhox,ssigx,rrhol,ssigl,RR,option,restr,per);

if maxiter == 0
    
    fval=floglik;
    xxnew=xx;
    
elseif maxiter > 0
    
    if optype == 1
        
        optionals = optimset('LevenbergMarquardt','on','Display','iter','TolFun',1e-8,'TolX',1e-8,'MaxFun',maxiter,'Diagnostics','on','MaxFunEvals',maxiter,'MaxIter',maxiter);
        [xxnew,fval,exitflag] = fminsearch(@ckm,xx,optionals,Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb, ...
            rhoz,rhox,sigx,rrhol,ssigl,RR,option,restr,per)
        
        if exitflag == 1
            disp('(estimatemodel) Optimization converged succesfully')
        elseif exitflag == 0 
            disp('(estimatemodel) Iteration limit reached')
        end
        
        if isnan(fval) == 1 | abs(imag(fval)) > .1e-10
            error('(estimatemodel) fatal function value not admissible')
        end
        
    elseif optype == 2
        
        optionn(4)= maxiter; %limit for the number of iterations (15000 by default)
        optionn(5)=5; %in order to print intermediate results
        
        [xxnew,fval,options] = solvopt(xx,'ckm',[],optionn,[],[],Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb, ...
            rhoz,rhox,sigx,rrhol,ssigl,RR,option,restr,per);
        
        
        if options(9) <0
            disp('(estimatemodel) abnormal termination of optimisation')
        end
        
        if isnan(fval) == 1 | abs(imag(fval)) > .1e-0 
            error('fatal (estimatemodel) optimized function value not a sensible number')
        end
        
    elseif optype == 3
        
        x_sa_lb = [-5.52 -6.2 -3 0 0 0 0]';%lower bound on parameter values
        x_sa_ub = [-3.9 -4.6 10 .02 .02 .02 .02]';%upper bound on parameter values
        
        x_sa = xx; %initial values of parameters to be estimated. it is column vector
        
        %set inputs for simulated annealing
        vm_sa = 5*x_sa; %step length vector. it should encompass the region of interest given starting value
        
        t_sa = 10; %initial temperature
        maxevl = 100000; %1000000000;
        ns_sa = 20;%20; %each element of x_sa is varied ns_sa times and the function value is evaluated each time. After all the evaluations for all the parameters have been done, each elemt of vm_sa is adjusted such that half of all function evaluations are accepted
        npar_sa = length(x_sa);
        nt_sa = 100; %max(100,5*npar_sa); %after nt_sa*ns_sa*npar_par evaluations have been made, there is a change in temperature
        rt_sa = 0.5;%0.85; %temperature reduction factor
        
        max_sa = 0; %=0 do minimisation; =1 do maximisation (note that the program performs maximisation. we set max_sa=0 because the value function (f) in ssloglik was set to -f, so we need to change the sign another time to revert to -(-f)=f
        eps_sa = 1.0e-6; %error tolerance
        neps_sa = 4; %value suggested by Goffe et al.
        c_sa = 2*ones(npar_sa,1); %controls the step length adjustment. suggested value =2
        iprint = 3; %=0, nothing printed; =1 summary results before each temperature reduction; =2 each new step length (vm); =3 each function evaluation. Suggested value =1
        
        [xxnew, fval, nacc_sa, nfcnev_sa, nobds_sa, ier_sa, t_sa, vm_sa] = ...
            simul_annealing('ckm',x_sa,max_sa,rt_sa,eps_sa,ns_sa,nt_sa,neps_sa,maxevl,x_sa_lb,x_sa_ub,c_sa,iprint,t_sa,vm_sa, ...
            Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb, ...
            rhoz,rhox,sigx,rrhol,ssigl,RR,option,restr,per); 
        
    elseif optype == 4
        
        H0=eye(length(xx));
        grad=[];
        crit=.1e-10;
        
        [fval,xxnew,gh,H,itct,fcount,retcodeh] = csminwel('ckm',xx,H0,grad,crit,maxiter,Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb, ...
            rhoz,rhox,sigx,rrhol,ssigl,RR,option,restr,per); 
        
        se=diag(H);
        if min(se) < 0
            disp('(estimatemodel) negative standard error')
        end
        
    end
    
end

[rhol,sigl,sigz,rhox,sigx,R] = map(xxnew,rrhol,ssigl,rrhox,ssigx,RR,restr);

if optype == 4;return;end

if stderr == 1
    %now go for the hessian, do the computations with untransformed variables
    xx(1)=sigz;
    
    if length(xxnew) > 1 & length(xxnew) ~= 5   
        xx(3)=rhol;
        xx(2)=sigl;
        if length(xx) > 3
            xx(4)=xxnew(4);
            xx(5)=xxnew(5);
            xx(6)=xxnew(6);
            xx(7)=xxnew(7);
        end
    end
    
    if length(xxnew) == 5
        
        xx(4)=rhox;
        xx(5)=sigx;
        
    end
    
    H = fdhess('ckm',xx,Y,theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,sigx,rhol,sigl,RR,option,restr,per);
    
    isp=1;
    n=size(H,1);
    if rank(H) < n
        disp('(estimatemodel) Hessian not full rank')
        isp=0;
    end
    
    if isp == 1
        iH=inv(H);
        se=diag(iH);
        if min(se) < 0
            disp('(estimatemodel) negative standard errors')
            isp=0;
        else
            se=sqrt(se);
        end
    end
    
    if isp == 0
        se=[];
    end
    
    
else
    
    se=[];
    H=[];
    
end