function [bbet,V,S,v,v1] = findAR(p,N,W,lam,J,f,sw,func,varargin)
%
% This program takes as input a moving average representation, and generates as output
% if sw==1, a p-th order VAR and various hp-filtered statistics both for
% the case when there are only technology shocks and the case where all
% shocks are on.
% if sw==0, only the hp-filtered statistics are reported.
%
% The moving average representation is in func, which
% must have arguments, (L,par1,....,parn), where L is the lag operator, treated
% as a complex variable in the program. Also, par1,....,parn is a list of 
% parameters that may be needed to compute the moving average.
% In particular:
%
%   Ytilde(t) = [D0+D1*L+D2*L^2+...]*eps(t), where Eeps(t)eps(t)'=W.
%
%  The vector, J, allows the user to select which of the variables in Ytilde(t) appear in Y(t).
%  That is, the J(1),...,J(end) entries of Ytilde(t) appear in Y(t). If
%  J=[], then J is automatically set to J=1,...,size(W,1);
%
% Let bbet = [A1,A2,...,Ap]. On output, one gets
%           
%                Y(t) = A1*Y(t-1) + A2*Y(t-2) + ... + Ap*Y(t-p) + u(t)
%
%                       where Eu(t)*u(t)'=V.
%
% The vectors, A1, ...., Ap are computed by solving the relevant set of
% Yule-Walker equations. These require as input the covariance function of 
% Y(t). This is computed by integrating the weighted spectrum of Y(t),
% using an N-point Riemann approximation.
%
% The calculations are done by potentially applying a band pass filter to
% the data first. The filter throws out data with period f periods and
% less. This is done as follows. The frequencies considered are
% omega(j)=2*pi*j/N, for j=0,1,...,N/2. The period in time units associated with any j is
% N/j, so we consider periods infinity corresponding to j=0, all the way to period 2, corresponding
% to j=N/2. So, if we drop data with periods f and less, we first compute
% j=round(N/f), and then ignore the spectral density associated with j greater than this. 
% If f=[], then all this is ignored. If the parameter f seems confusing,
% simply set f=[] on input.
%
% The vector, v, has the same dimension as W, and contains the variance of
% (HP-filtered log output), (actual hours), (Hp-filtered log investment),
% (HP-filtered log consumption) in that order. 
% In addition, the 5-th element is the covariance between HP-filtered labor productivity
% and hours, the variance of HP-filtered log productivity and the variance
% of HP-filtered, log hours.
%
% The value of lambda, lam, must be supplied on input.
%
%  An example was considered, with Y(t)=.98*Y(t-1)+u(t), V=1. It was found
% that excellent accuracy was obtained with N=500. With N=100 and p=9,
% accuracy to two digits occured.

% the following is the HP filter in frequency domain:
%   HP(L) = H*(1-L)(1-L)(1-1/L)(1-1/L)/[(1-g1*L-g2*L^2)(1-g1/L-g2/L^2)],
%   where H = -g2.

if isempty(J) == 1
    J=1:size(W,1);
end
  
[g1,g2]=hpspect(lam);
H=-g2;
W1=zeros(size(W));
W1(1,1)=W(1,1);

D=feval(func,1,varargin{:}); 
S0=D*W*D';
S01=D*W1*D';

nvar=length(J);
nnvar=size(D,1);
i=sqrt(-1);
%first, compute the covariance function by integrating the weighted
%spectral density

S(:,:,1)=S0;

v=zeros(6,1);
v1=zeros(6,1);

if isempty(f) == 1
    JJ=N;
else
    JJ=round(N/f);
end

if JJ > N/2
    a=ones(1,N/2+1);
else
    a=zeros(1,N/2+1);
    a([1:JJ])=ones(1,JJ);
end

v=zeros(7,1);
v1=zeros(7,1);

for ii = 0:1:(N/2)
    
    omeg=2*pi*ii/N;
    L=exp(-i*omeg);
    D=feval(func,L,varargin{:});
    S(:,:,ii+1)=D*W*(D');
    S1=D*W1*(D');
    
    HP1=H*(1-L)*(1-1/L)*(1-1/L)/((1-g1*L-g2*L^2)*(1-g1/L-g2/L^2));%this is hp filter for variable that's already been differenced
    HP2=H*(1-L)*(1-L)*(1-1/L)*(1-1/L)/((1-g1*L-g2*L^2)*(1-g1/L-g2/L^2));
    hp1=HP1*conj(HP1);
    hp2=HP2*conj(HP2);
    
    %log output = (Y(1,t)+(1-L)*Y(2,t))/(1-L) = Py(L)*Y(t)/(1-L)
    %log investment = Y(3,t)+log output = Pi(L)'*Y(t)/(1-L)
    %                 Pi(L) = [1 1-L 1-L 0]'
    %log consumption = Y(4,t) + log output = Pc(L)'*Y(t)/(1-L)
    %                 Pc(L) = [1 1-L 0 1-L]';
    %log labor productivity = Y(1,t)/(1-L) 
    
    Py=zeros(1,nnvar);
    Pc=zeros(1,nnvar);
    Pi=zeros(1,nnvar);
    
    Py(1)=1;
    Py(2)=1-L;
    vy=(Py*S(:,:,ii+1)*Py')*hp1;
    vy1=(Py*S1*Py')*hp1;
    Pi(1)=1;
    Pi(2)=1-L;
    Pi(3)=1-L;
    if nnvar < 3        
        Pi=Pi([1:nnvar]);
    end
    vi=(Pi*S(:,:,ii+1)*Pi')*hp1;
    vi1=(Pi*S1*Pi')*hp1;
    Pc(1)=1;
    Pc(2)=1-L;
    Pc(4)=1-L;
    if nnvar < 4
        Pc=Pc([1:nnvar]);
    end
    vc=(Pc*S(:,:,ii+1)*Pc')*hp1;
    vc1=(Pc*S1*Pc')*hp1;
    
    vh=S(2,2,ii+1)*hp2;
    vh1=S1(2,2)*hp2;

    ck=2;
    if ii == N/2 | ii == 0 ;ck=1;end

    v(1)=v(1)+ck*real(vy)/N;%computing variance of HP-filtered output
    v(2)=v(2)+ck*real(S(2,2,ii+1))/N;%computing variance of log employment
    v(3)=v(3)+ck*real(vi)/N;
    v(4)=v(4)+ck*real(vc)/N;
    v(5)=v(5)+ck*real(S(1,2,ii+1)*HP1*conj(HP2))/N;%covariance between labor productivity and hours
    v(6)=v(6)+ck*real(S(1,1,ii+1)*hp1)/N;%variance of labor productivity
    v(7)=v(7)+ck*real(vh)/N;%variance of hp-filtered labor
    
    v1(1)=v1(1)+ck*real(vy1)/N;%computing variance of HP-filtered output
    v1(2)=v1(2)+ck*real(S1(2,2))/N;%computing variance of log employment
    v1(3)=v1(3)+ck*real(vi1)/N;
    v1(4)=v1(4)+ck*real(vc1)/N;
    v1(5)=v1(5)+ck*real(S1(1,2)*HP1*conj(HP2))/N;%covariance between labor productivity and hours
    v1(6)=v1(6)+ck*real(S1(1,1)*hp1)/N;%variance of labor productivity
    v1(7)=v1(7)+ck*real(vh1)/N;%variance of hp-filtered labor
    
end

if sw ~= 1
    bbet=[];
    V=[];
else
    
    for lag = 1:p+1
        lg=lag-1;
        D=feval(func,1,varargin{:});
        C(:,:,lag)=a(1)*real(S(J,J,1))/N;
        for ii = 1:(N/2)-1
            omeg=2*pi*ii/N;
            L=exp(-i*omeg);
            D=feval(func,L,varargin{:});  
            C(:,:,lag)=C(:,:,lag)+a(ii+1)*real(S(J,J,ii+1)*exp(i*omeg*lg))*2/N;
        end
        D=feval(func,-1,varargin{:});    
        C(:,:,lag)=C(:,:,lag)+a(N/2+1)*real(S(J,J,N/2+1)*exp(i*pi*lg))/N;
    end
    
    for ii = 1:p
        d([1:nvar],[1:nvar]+(ii-1)*nvar)=C(:,:,ii+1);
    end
    
    X=zeros(nvar*p,nvar*p);
    for ii = 1:p
        for jj = 1:p-(ii-1);
            X(nvar*(ii-1)+[1:nvar],nvar*(ii-1)+nvar*(jj-1)+[1:nvar])=C(:,:,jj);
        end
    end
    
    for ii = 1:nvar*p
        for jj = 1:ii-1
            X(ii,jj)=X(jj,ii);
        end
    end
    
    bbet=d/X;
    
    V=C(:,:,1);
    for ii = 1:p
        V=V-bbet(:,(ii-1)*nvar+[1:nvar])*C(:,:,ii+1)';
    end
    
end