function [A,B,h0,h1,P] = linearizeandsolve(k,l,y,c,taul,taux,gamma,theta,delta,muz,psi,lb,beta, ...
    rhoz,rhox,rhol,chk)
%This returns the linearized representation of the intertemporal and intratemporal euler
%equations. The coefficients in the linearized representations are computed
%analytically.
%  khat(t+1) = A*khat(t) + B*s(t)
%  lhat(t) = h0*khat(t) + h1*s(t) 
%  s(t) = (mu(t) taul(t)-taul taux(t)-taux)'
%  s(t) = P*s(t-1) + error
% Here, taul is the tax rate on labor, taul is the ad valorem tax on
% capital and mu is the growth rate of technology, in deviation from mean.

%If chk = 1, all calculations are checked by numerical differentiation.

x(1)=k;
x(2)=k;
x(3)=l;
x(4)=0;
x(5)=taul;

%verify that the intratemporal euler equation is satisfied...
[ff] = intra(x,gamma,theta,delta,muz,psi,lb);
if abs(ff) > .1e-10
    error('fatal (linearizeandsolve) intratemporal euler equation not satisfied')
end

if chk == 1
    %take a taylor series expansion of the intratemporal euler equation (if chk -1)
    [chkV1]=derivative(x,'intra',gamma,theta,delta,muz,psi,lb);
end

%do for the intertemporal euler equation what we just did to the
%intratemporal one...
x(1)=k;
x(2)=k;
x(3)=k;
x(4)=l;
x(5)=l;
x(6)=0;
x(7)=0;
x(8)=taux;
x(9)=taux;

[ff] = inter(x,gamma,theta,delta,muz,beta,c,taux);

if chk == 1
    [chkV2]=derivative(x,'inter',gamma,theta,delta,muz,beta,c,taux);    
end

if abs(ff) > .1e-10
    error('fatal (linearizeandsolve) intertemporal euler equation not satisfied')
end

%log-linear expansion of resource constraint:
% chat(t) = cl*lhat(t) + ck*khat(t) + ckp*khat(t+1) + cmu*dmu(t)

cl=(y/c)*(1-theta);
ck=(y/c)*theta+(1-delta)*(k/c)*exp(-muz);
ckp=-(1+gamma)*k/c;
cmu=-(y/c)*theta-(1-delta)*exp(-muz)*(k/c);

%V1 is linear expansion of static euler equation:
V1=[(ck-theta)/k ckp/k (cl+l/(lb-l)+theta)/l cmu+theta 1/(1-taul)]*psi*c/(lb-l);
if chk == 1
    if max(abs(V1./chkV1-1)) > 10e-6
        error('fatal (lineareandsolve) intratemporal euler equation wrong')
    end
end

if chk == 1
    x=[];
    x(1)=l;
    x(2)=k;
    x(3)=k;
    x(4)=0;
    [C]=derivative(x,'chkcons',gamma,theta,delta,muz,c);
    C=C.*[l k k 1];
    chkc=[cl ck ckp cmu];
    if max(abs(C./chkc-1)) > 8e-6
        error('fatal (linearizeandsolve) failed to linearize resource constraint properly')
    end    
end

%log linear expansion of scaled return on capital, r(t+1):
% rhat(t+1) = rkp*khat(t+1) + rlp*lhat(t+1) + rtauxp*dtaux(t+1) + rmup*dmu(t+1)

disc=exp(muz)*(1+taux)/beta;
rkp=theta*(theta-1)*((exp(-muz)*(k/l))^(theta-1))/disc;
rlp=-rkp;
rmup=-rkp;
rtauxp=(1-delta)/disc;

if chk == 1
    x=[];
    x(1)=k;
    x(2)=l;
    x(3)=0;
    x(4)=taux;
    [rr]=derivative(x,'chkrhat',theta,muz,delta,taux,beta);
    rr=rr.*[k l 1 1];
    tt=[1 2 4];
    chkrr=[rkp rlp rmup rtauxp];
    %The third term seems to be less accurately estimated by numerical
    %differentiation. That's why this test is less tight
    if max(abs(rr(tt)./chkrr(tt)-1)) > 8e-6 | (abs(rr(3)./chkrr(3)-1)) > .3e-3
        error('fatal (linearizeandsolve) linearization of return to capital not accurate')
    end
end

V2(1)=-ck/k;
V2(2)=(-ckp+ck-rkp)/k;
V2(3)=ckp/k;
V2(4)=-cl/l;
V2(5)=(cl-rlp)/l;
V2(6)=-cmu;
V2(7)=1+cmu-rmup;
V2(8)=1/(1+taux);
V2(9)=-rtauxp;

if chk == 1
    if max(abs(chkV2./V2-1)) > 10e-6
        error('fatal (linearizeandsolve) linear expansion of intertemporal Euler equation failed')
    end
end

%get the shortened intertemporal euler equation....
%al0*kpp + al1*kp + al2*k + bet0*sp + bet1*s

al2=(V2(1)-V2(4)*V1(1)/V1(3))*k;
al1=(V2(2)-V2(4)*V1(2)/V1(3)-V2(5)*V1(1)/V1(3))*k;
al0=(V2(3)-V2(5)*V1(2)/V1(3))*k;

bet1=[V2(6)-V2(4)*V1(4)/V1(3) -V2(4)*V1(5)/V1(3) V2(8)];
bet0=[V2(7)-V2(5)*V1(4)/V1(3) -V2(5)*V1(5)/V1(3) V2(9)];

if chk == 1%this checks the version of the intertemporal euler with the resource constraint
    %used to substitute out for consumption and the intratemporal euler
    %equation used to substitute out for labor.
    [ff] = findlabor(l,k,k,0,taul,gamma,theta,delta,muz,lb,psi);
    [lx,cx] = ffindlabor(k,k,0,taul,gamma,theta,delta,muz,lb,psi);
    if abs(lx-l) > .1e-10 | abs(cx-c) > .1e-10
        error('fatal (linearizeandsolve) program for computing consumption and labor using intra failed')
    end
    x=[];
    x(1)=k;
    x(2)=k;
    x(3)=k;
    x(4)=0;
    x(5)=taul;
    x(6)=taux;
    x(7)=0;
    x(8)=taul;
    x(9)=taux;    
    [ffx] = intertemporal(x,gamma,theta,delta,muz,lb,psi,beta,c,taux);
    if abs(ffx) > .1e-10
        error('fatal (linearizeandsolve) reduced intertemporal euler equation not satisfied')
    end    
    [Vk]=derivative(x,'intertemporal',gamma,theta,delta,muz,lb,psi,beta,c,taux);
    chkVk=[Vk([1:3])*k Vk([4:end])];
    VVk=[al2 al1 al0 bet1 bet0];
    if max(abs(chkVk./VVk-1)) > 7e-5
        error('fatal (linearizeandsolve) approximation of intertemporal euler equation failed')
    end
end

%solve for the capital policy rule
dscr=(-4*al0*al2+al1^2);
if dscr < 0
    error('fatal (linearizeandsolve) discriminant negative')
end
A=-al1/(2*al0)+(1/(2*al0))*sqrt(dscr);
if abs(A) > 1
    A=-al1/(2*al0)-(1/(2*al0))*sqrt(dscr);
end
if abs(A) > 1
    error('fatal (linearizeandsolve) no stable eigenvalue found')
end

P=zeros(3,3);
P(1,1)=rhoz;
P(2,2)=rhol;
P(3,3)=rhox;

%solve for the hours policy rule
B=-(bet0*P+bet1)/((al0*A+al1)*eye(3)+al0*P);

q=al0*A^2+al1*A+al2;
F=al0*A*B+al0*B*P+al1*B+bet0*P+bet1;
if max(max(abs(F))) > .1e-10 | abs(q) > .1e-10
    error('fatal (linearizeandsolve) either scalar, A, or B matrix do not satisfy accuracy condition')
end

h0=-V1(1)*k/(V1(3)*l)-V1(2)*k*A/(V1(3)*l);
h1=[-V1(4)/(V1(3)*l) -V1(5)/(V1(3)*l) 0]-(V1(2)*k/(V1(3)*l))*B;