function [alpha_0,alpha_1,alpha_2,beta_0,beta_1] = ...
    makealphabeta(k,l,y,c,taul,taux,gamma,theta,delta,muz,psi,lb,beta,rhoz,rhox,rhol,sig,chk)
%This linearizes the intra and inter temporal euler equation and puts them into the
%canonical form:
%
%        alpha_0*z(t+1)+alpha_1*z(t)+alpha_2*z(t-1) + beta_0*s(t+1)+beta_1*s(t)
%
%  where
%
%           z(t) = [khat(t+1) lhat(t)]'
%
% If chk = 1, all linearizations are checked by numerical differentiation.

x(1)=k;
x(2)=k;
x(3)=l;
x(4)=0;
x(5)=taul;

%verify that the intratemporal euler equation is satisfied...
[ff] = intra(x,gamma,theta,delta,muz,psi,lb,sig);
if abs(ff) > .1e-10
    error('fatal (linearizeandsolve) intratemporal euler equation not satisfied')
end

if chk == 1
    %take a taylor series expansion of the intratemporal euler equation (if chk -1)
    [chkV1]=derivative(x,'intra',gamma,theta,delta,muz,psi,lb,sig);
end

%do for the intertemporal euler equation what we just did to the
%intratemporal one...
x(1)=k;
x(2)=k;
x(3)=k;
x(4)=l;
x(5)=l;
x(6)=0;
x(7)=0;
x(8)=taux;
x(9)=taux;

[ff] = inter(x,gamma,theta,delta,muz,beta,c,taux);

if chk == 1
    [chkV2]=derivative(x,'inter',gamma,theta,delta,muz,beta,c,taux);    
end

if abs(ff) > .1e-10
    error('fatal (linearizeandsolve) intertemporal euler equation not satisfied')
end

%log-linear expansion of resource constraint:
% chat(t) = cl*lhat(t) + ck*khat(t) + ckp*khat(t+1) + cmu*dmu(t)

cl=(y/c)*(1-theta);
ck=(y/c)*theta+(1-delta)*(k/c)*exp(-muz);
ckp=-(1+gamma)*k/c;
cmu=-(y/c)*theta-(1-delta)*exp(-muz)*(k/c);

%V1 is linear expansion of static euler equation:
V1=[(ck-theta)/k ckp/k (cl+sig*l/(lb-l)+theta)/l cmu+theta 1/(1-taul)]*psi*c/(lb-l);
if chk == 1
    aa=V1./chkV1;
    aa=aa/aa(1);
    if max(abs(aa-1)) > 10e-6
        error('fatal (lineareandsolve) intratemporal euler equation wrong')
    end
end

if chk == 1
    x=[];
    x(1)=l;
    x(2)=k;
    x(3)=k;
    x(4)=0;
    [C]=derivative(x,'chkcons',gamma,theta,delta,muz,c);
    C=C.*[l k k 1];
    chkc=[cl ck ckp cmu];
    if max(abs(C./chkc-1)) > 8e-6
        error('fatal (linearizeandsolve) failed to linearize resource constraint properly')
    end    
end

%log linear expansion of scaled return on capital, r(t+1):
% rhat(t+1) = rkp*khat(t+1) + rlp*lhat(t+1) + rtauxp*dtaux(t+1) + rmup*dmu(t+1)

disc=exp(muz)*(1+taux)/beta;
rkp=theta*(theta-1)*((exp(-muz)*(k/l))^(theta-1))/disc;
rlp=-rkp;
rmup=-rkp;
rtauxp=(1-delta)/disc;

if chk == 1
    x=[];
    x(1)=k;
    x(2)=l;
    x(3)=0;
    x(4)=taux;
    [rr]=derivative(x,'chkrhat',theta,muz,delta,taux,beta);
    rr=rr.*[k l 1 1];
    tt=[1 2 4];
    chkrr=[rkp rlp rmup rtauxp];
    %The third term seems to be less accurately estimated by numerical
    %differentiation. That's why this test is less tight
    if max(abs(rr(tt)./chkrr(tt)-1)) > 7e-5 | (abs(rr(3)./chkrr(3)-1)) > .3e-3
        error('fatal (linearizeandsolve) linearization of return to capital not accurate')
    end
end

V2(1)=-ck/k;
V2(2)=(-ckp+ck-rkp)/k;
V2(3)=ckp/k;
V2(4)=-cl/l;
V2(5)=(cl-rlp)/l;
V2(6)=-cmu;
V2(7)=1+cmu-rmup;
V2(8)=1/(1+taux);
V2(9)=-rtauxp;

if chk == 1
    if max(abs(chkV2./V2-1)) > 10e-6
        error('fatal (linearizeandsolve) linear expansion of intertemporal Euler equation failed')
    end
end

alpha_0=zeros(2,2);
alpha_1=zeros(2,2);
alpha_2=zeros(2,2);
beta_0=zeros(2,3);
beta_1=zeros(2,3);

alpha_0(1,1)=V2(3)*k;
alpha_0(1,2)=V2(5)*l;
alpha_1(1,1)=V2(2)*k;
alpha_1(1,2)=V2(4)*l;
alpha_1(2,1)=V1(2)*k;
alpha_1(2,2)=V1(3)*l;
alpha_2(1,1)=V2(1)*k;
alpha_2(2,1)=V1(1)*k;

beta_0(1,1)=V2(7);
beta_0(1,3)=V2(9);
beta_1(1,1)=V2(6);
beta_1(1,3)=V2(8);
beta_1(2,1)=V1(4);
beta_1(2,2)=V1(5);
