function [L,Sbar,P0,P,Q,A,B,C,param,innov,MY,DY,MX] = mleq(Theta,dummy,ver);
%MLE     Log-likelihood function for the standard growth model with
%        fluctuations in four ``wedges'': z, taul, taux, and g.
%        In this version of the model, technology has a unit root.
%        See appendix1.tex for details.
%

%        Ellen R. McGrattan, 10-29-02 
%        Revised, ERM, 9-14-04

global ZVAR    % = [Growth in output per capita, 
               %    Growth in investment per capita,
               %    Hours per capita,
               %    Growth in government spending per capita]
%---------------------------------------------------------------------
% 1. Default parameters for 
%    gn, gz, beta, delta, psi, sigma, theta, Sbar, P, Q, D, R
%

gn      = (1.01)^(1/4)-1;
gz      = (1.02)^(1/4)-1;
beta    = .98^(1/4);
delta   = 1-(1-.06)^(1/4);
psi     = 2.5;
sigma   = 1.000001;
theta   = .33;
P       = .95*eye(4);
Sbar    = [.004;.25;.3;log(.17)];
P0      = (eye(4)-P)*Sbar;
nn=4;
if ver >= 3;nn=3;end
Q       = zeros(4);
D       = zeros(nn);
R       = 1e-4*eye(nn);

if ver == 2 | ver == 4
    for ii = 1:nn
        R(ii,ii)=Theta(5+ii)^2;
    end
    Theta=Theta([1:5]);
end

%---------------------------------------------------------------------
% 2. Scale Theta. 

Tsize = ones(length(Theta),1);  
Theta = diag(Tsize)*Theta(:);
%---------------------------------------------------------------------

%---------------------------------------------------------------------
% 3. Assign elements of Theta to underlying parameters.  Use the vector
%    ind to indicate which parameters are to be estimated and which
%    are to be fixed in estimation.  For example, if the vector of
%    parameters is [beta,gamma,delta] and gamma is to be fixed during
%    estimation then set:
%       beta  = Theta(1);
%       delta = Theta(2);
%       ind   = [1,0,1];
%
%    For the current example, the ordering of the parameter vector is:
%       gn, gz, beta, delta, psi, sigma, theta,  Sbar,
%       P(:),  Q(:),  D(:),  R(:)
%        1   2     3      4    5      6      7   8-11 
%      12-27  28-43  44-59  60-75

ind     = [0,0,0,0,0,0,0, 1,1,0,0, ...
           0,0,0,0, 0,1,0,0, 0,0,0,0, 0,0,0,0, ...
           1,0,0,0, 0,1,0,0, 0,0,0,0, 0,0,0,0, ...
           0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, ...
           0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0];
P       = zeros(4);
Q       = zeros(4);
Sbar(1) = Theta(1);
Sbar(2) = Theta(2);
Sbar(3) = .3;
Sbar(4) = log(.17);
P(2,2)  = Theta(3);
Q(1,1)  = Theta(4);
Q(2,2)  = Theta(5);
P0      = (eye(4)-P)*Sbar;
param   = [gn;gz;beta;delta;psi;sigma;theta];
%---------------------------------------------------------------------
% 4. Put bounds on Theta if there are values of the parameters for
%    which L cannot be computed. (Optional)

%    recall ordering: 
%       gn, gz, beta, delta, psi, sigma, theta, Sbar
%       P(:),  Q(:),  D(:),  R(:)
%        1   2     3      4    5      6      7  8-11
%      12-27  28-43  44-59  60-75
%          
%---------------------------------------------------------------------
Ub  = [.1,.1,.99999,1,3,5,.5,  1,1,1,1,  ....
        1.40, 1.40, 1.40, 1.40, 1.40, 1.40, 1.40, 1.40, 1.40,  ...
              1.40, 1.40, 1.40, 1.40, 1.40, 1.40, 1.40,  ...
        1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000,  ...
              1000, 1000, 1000, 1000, 1000, 1000, 1000,  ...
        1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000,  ...
              1000, 1000, 1000, 1000, 1000, 1000, 1000,  ...
        1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000,  ...
              1000, 1000, 1000, 1000, 1000, 1000, 1000]';

Lb  = [0,0,0,0,0,0,0, 0,-1,-1,-1, ...
       -.999,-.999,-.999,-.999,-.999,-.999,-.999,-.999,-.999,  ...
             -.999,-.999,-.999,-.999,-.999,-.999,-.999,  ...
       -1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,  ...
             -1000,-1000,-1000,-1000,-1000,-1000,-1000,  ...
       -1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,  ...
             -1000,-1000,-1000,-1000,-1000,-1000,-1000,  ...
       -1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,-1000,  ...
             -1000,-1000,-1000,-1000,-1000,-1000,-1000]';
Ub  = Ub(find(ind));
Lb  = Lb(find(ind));
if (any(Theta>Ub) | any(Theta<Lb)); 
  L = 1e+20; dL=1e+20*ones(size(Theta));
  return
end;
penalty = 0*500000*max(max(abs(eig(P)))-.995,0).^2;

%---------------------------------------------------------------------


%---------------------------------------------------------------------
% 5.  Compute equilibrium
%---------------------------------------------------------------------

%
% 5a. Steady state:
%
tem        = (eye(4)-P)\P0;
zs         = exp(tem(1));
tauls      = tem(2);
tauxs      = tem(3);
gs         = exp(tem(4));
kls        = ((1+tauxs)*(1-beta*zs^(-sigma)*(1-delta))/ ...
              (beta*zs^(-sigma)*theta))^(1/(theta-1))*zs;
A          = kls^(theta-1)*zs^(-theta)-(1+gn)+(1-delta)/zs;
B          = (1-tauls)*(1-theta)*kls^theta*zs^(-theta)/psi;
ks         = (B+gs)/(A+B/kls);
cs         = A*ks-gs;
ls         = ks/kls;
ys         = (ks/zs)^theta*ls^(1-theta);
xs         = ys-cs-gs;
X0s        = [log(ks);log(zs);tauls;tauxs;log(gs);1];
X0         = [X0s;X0s];
Y0         = [log(zs);log(zs);log(ls);log(zs)];

%
% 5b. Call subroutine with residuals:
%

Z          = [log(ks);log(ks);log(ks);log(zs);log(zs);tauls;tauls;
              tauxs;tauxs;log(gs);log(gs)];
del        = max(abs(Z)*1e-5,1e-8);
for i=1:11;
  Zp       = Z;
  Zm       = Z;
  Zp(i)    = Z(i)+del(i);
  Zm(i)    = Z(i)-del(i);
  dR(i,1)  = (res_wedge_ur(Zp,param)-res_wedge_ur(Zm,param))/(2*del(i));
end;

%
% 5c. Solution:  log k[t+1] = gamma0 + gammak* log k[t] + gamma* S[t]
%

a0         = dR(1);
a1         = dR(2);
a2         = dR(3);
b0         = dR(4:2:11)';
b1         = dR(5:2:11)';
tem        = roots([a0,a1,a2]);
gammak     = tem(find(abs(tem)<1));
gamma      = -((a0*gammak+a1)*eye(4)+a0*P')\(b0*P+b1)';
gamma0     = (1-gammak)*log(ks)-gamma'*[log(zs);tauls;tauxs;log(gs)];
Gamma      = [gammak;gamma;gamma0];

%
% 5d. State-space system:   X[t+1] = A X[t] + B eps[t+1]
%                           Y[t]   = C X[t] + ome[t]
%                           ome[t] = D ome[t-1] + eta[t],  E eta eta' ~ N(0,R)
%


philh      =-(psi*ys*(1-theta)+(1-theta)*(1-tauls)*ys*(1-ls)/ls*theta+ ...
             (1-theta)*(1-tauls)*ys);
philk      = (psi*ys*theta+psi*(1-delta)*ks/zs- ...
             (1-theta)*(1-tauls)*ys*(1-ls)/ls *theta)/philh;
philz      = (psi*ys*(-theta)-psi*(1-delta)*ks/zs+ ...
             (1-theta)*theta*(1-tauls)*ys*(1-ls)/ls)/philh;
phill      = ((1-theta)*(1-tauls)*ys*(1-ls)/ls *(1/(1-tauls)))/philh;
philg      = (-psi*gs)/philh;
philkp     = (-psi*(1+gn)*ks)/philh;
phiyk      = theta+(1-theta)*philk;
phiyz      = (1-theta)*philz-theta;
phiyl      = (1-theta)*phill;
phiyg      = (1-theta)*philg;
phiykp     = (1-theta)*philkp;
phixk      = -ks/(zs*xs)*(1-delta);
phixkp     = ks/xs*(1+gn);
phixz      = ks/(zs*xs)*(1-delta);

As    = [ gammak,                         gamma', gamma0;
         [0;0;0;0],            P,                     P0;
               0,      0,      0,      0,      0,      1];
Bs    = [ 0,0,0,0;
            Q;
          0,0,0,0];
A     = [As,zeros(6);
         eye(6),zeros(6)];
B     = [Bs;
         zeros(6,4)];

phiy0 = log(ys)-phiyk*log(ks)-phiyz*log(zs)-phiyl*tauls-phiykp*log(ks);
phix0 = log(xs)-phixk*log(ks)-phixz*log(zs)-phixkp*log(ks);
phil0 = log(ls)-philk*log(ks)-philz*log(zs)-phill*tauls-philg*log(gs)- ...
                philkp*log(ks);
C     = [ [phiyk-phiykp,1+phiyz,phiyl,     0,phiyg,phiy0, ...
           -phiyk,-phiyz,-phiyl,    0,-phiyg,-phiy0]+ ....
            phiykp*[Gamma(1:6)',zeros(1,6)];
          [phixk-phixkp,1+phixz,    0,     0,    0,phix0, ...
           -phixk,-phixz,     0,    0,     0,-phix0]+ ....
            phixkp*[Gamma(1:6)',zeros(1,6)];
          [philk,philz,phill,    0,philg,phil0,0,0,0,0,0,0]+ ....
            philkp*[Gamma(1:6)',zeros(1,6)];
          0,1,0,0,1,0,0,0,0,0,-1,0];

 if ver >= 3
     C=C([1:3],:);
 end
%---------------------------------------------------------------------
% 6. Specify observables. 
%

T          = length(ZVAR);
Y          = log(ZVAR);
Ybar       = Y(2:T,:)-Y(1:T-1,:)*D';
T          = T-1;
Cbar       = C*A-D*C;
Rbar       = R+C*B*B'*C';
[K,Sigma]  = kfilter(A,Cbar,B*B',Rbar,B*B'*C');
Omega      = Rbar+Cbar*Sigma*Cbar';
Omegai     = inv(Omega);
Xt(1,:)    = X0';
innov(1,:) = Ybar(1,:)-X0'*Cbar';
for i=2:T;
  Xt(i,:)    = Xt(i-1,:)*A'+innov(i-1,:)*K';
  innov(i,:) = Ybar(i,:)-Xt(i,:)*Cbar';
end;
sum1       = innov(1:T,:)'*innov(1:T,:)/T;

%---------------------------------------------------------------------
% 7. Likelihood function
L          = 0.5*(T*(log(det(Omega))+trace(Omegai*sum1))+penalty);

%---------------------------------------------------------------------
% 8. Additional information
MY         = exp(Xt*Cbar');
DY         = exp(Ybar);
MX         = [exp(Xt(:,1:2)),Xt(:,3:4),exp(Xt(:,5))];

%---------------------------------------------------------------------
% NOTE: Original fixed parameters based on BCA are given below
%gn      = (1.015)^(1/4)-1;
%gz      = (1.016)^(1/4)-1;
%beta    = .9722^(1/4);
%delta   = 1-(1-.0464)^(1/4);
%psi     = 2.24;
%sigma   = 1.000001;
%theta   = .35;
%Sbar(3) = .3;
%Sbar(4) = log(.18);
