function [M,B,C,Bsum,Bsumchk] ...
   = sargentvillaverde(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,sigz,sigl,sigx,P,sig,tau,J,T,def);

%INPUT:
%This takes model parameters (for the model, see solvemodel.m, except curvature in 
%utility of labor is allowed, as controlled by sig) as input. 
%
%The fundamental economic shocks are technology (std dev controlled by sigz), and one or
%both of preferences (if sigl > 0) and capital taxes (if sigx > 0). 
%If either of these two standard deviations are zero, then there are
%two shocks total: technology and whichever of preferences and 
%capital taxes have a positive std dev.
%
%There are two possible definitions of the observer equation:
%
%if def = 1 (this is the default, in case def is not included in the input list)
%
%  X(t) = [dlog(a(t)) log(l(t)) log(c(t)/y(t)) log(i(t)/y(t))]'
%
%if def = 2:
%
%  X(t) = [dlog(y(t)) log(l(t)) dlog(c(t)) dlog(i(t))]'
%
%The vector, J, picks out the elements of X(t) you want in the observer
%equation. Must include as first two elements, 1 and 2. 
%If J = [1 2 3], then the observed data are the first three elements of
%X(t). 
%
%The number of elements in J must be the same as the number of fundamental
%economic shocks.
%
%OUTPUT:
%The program checks whether the fundamental economic shocks lie in the
%space of current and past observed data. A message is generated only if
%invertibility fails.
%
%M, the crucial matrix in the VAR(infinity) representation
%
%B, B(:,:,1),...,B(:,:,T), the VAR lag 1, lag 2, ... lag T matrices in
%var(infinity representation) (the 100 can be changed to any other positive
%integer, but this number is hard-wired in the program).
%
%C, The matrix characterizing the impact on X of the fundamental (variance unity) shocks
%   in the var(infinity representation)
%
%Bsum, the actual infinite sum over all B's
%
%Bsumchk, the sum of the first T B's

if nargin < 21
    def=1;
end

if length(J) < 2 | length(J) > 3
    error('fatal (sargentvillaverde) wrong number of variables in observer vector')
end
if J(1) ~= 1
    error('fatal (sargentvillaverde) first variable in observer must be productivity')
end
if J(2) ~= 2
    error('fatal (sargentvillaverde) second variable in observer must be hours')
end

if abs(sigz) < .1e-10
    error('fatal (sargentvillaverde) variance of technology shock too small')
end

%JJ selects which fundamental economic shocks we're working with. You must work with
%the technology shock and hours worked. You can work with one, both or none
%of the other two shocks.

JJ=[1 2 3];
if abs(sigl) < .1e-10
    JJ=[1 3];
elseif abs(sigx) < .1e-10
    JJ=[1 2];
end

if length(J) ~= length(JJ)
     error('fatal (sargentvillaverde) wrong number of variables in observer equation, relative to shocks')
end


[H,F,D,Qt,n] = stateobserver(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,J,JJ,def);


M=(eye(2+2*n)-D*inv(H*D)*H)*F;
rr=eig(M);
if max(abs(rr)) > 1
    error('fatal (sargentvillaverde) failure of invertibility')
end

tt=[1:length(J)];
ss=zeros(length(J),length(J));
if nargin < 20
    T=100;
end

for ii = 1:T
    BB=H*F*(M^(ii-1))*D*inv(H*D);
    B(tt,tt,ii)=BB;
    ss=ss+BB;
end

Bsumchk=ss;
Bsum=H*F*inv(eye(2+2*n)-M)*D*inv(H*D);
C=H*D;