function [A,B,P,a0,a1,a2,b0,b1,V,y,l,c,k,psi] = solution(theta,muz,taux,beta,delta,gamma, ...
    taul,psi,lb,rhoz,rhox,rhol,tau,sigz,sigl,sigx,sig,chk,def,ll)
%
%  This computes the solution to the model described in solvemodel.m
%
%  let z(t) = [khat(t+1) lhat(t)]', X(t) = [dlog(y/l) log(l) log(I/Y) log(C/Y)]' 
%
%  Then, 
%
%               z(t) = A*z(t-1) + B*s(t)
%
%               X(t) = a0 + a1*khat(t) + a2*khat(t-1) + b0*s(t) + b1*s(t-1)
%
%               s(t) = P*s(t-1) + eps(t), Eeps(t)*eps(t)' = V
%
%  tau=[tau1,tau2], where tau1 and tau2 are each 3,1 vectors. tau1 pertains to the 
%  investment euler equation and khat(t+1) decision. tau2 pertains to lhat(t).
%  If all elements of taui are unity, then the period t realization of each shock is observed 
%  before the period t choice of the ith variable. The shocks are ordered
%  by technology growth (1), labor tax (2), investment tax (3). If the jth
%  element of taui is zero, then the period t realization of the jth shock
%  is not observed when the period t value of the ith variable is selected.
%  The parameter, def, indicates how X(t) is constructed. See rform.m
%
%  The right dimension of B, b0 and b1 is 6 if any element of tau is zero.
%  Otherwise the right dimensions are 3. 
%
%  if there are 19 input parameters, then the last one, ll, is the desired
%  steady state value of employment. The program returns the value of psi
%  that rationalizes ll.



%get the steady state:
[y,l,c,k,A] = sstate(theta,muz,taux,beta,delta,gamma,taul,psi,lb,sig);
if nargin == 20
    if ll < 0 | ll > lb
        error('fatal (solution) trying to set steady state labor to an inadmissible value')
    end
    cc=(1-taul)*(1-theta)*(((k/l)*exp(-muz))^theta)*(((lb-ll)^sig)/ll);
    psi=cc/A;
    [y,l,c,k,A] = sstate(theta,muz,taux,beta,delta,gamma,taul,psi,lb,sig);
end

%get the linearized euler equations
[alpha_0,alpha_1,alpha_2,beta_0,beta_1] = ...
    makealphabeta(k,l,y,c,taul,taux,gamma,theta,delta,muz,psi,lb,beta,rhoz,rhox,rhol,sig,chk);

alphas=[alpha_0,alpha_1,alpha_2];
%solve for A:
AIM=0;
if AIM == 1
    %New version of AIM, produce same answer but much faster
    condn  = 10^(-10); %condition used by AIM if a matrix does not invert, not important here
    uprbnd = 0.99999;  %largest eigenvalue of the feedback matrix, allowed to be slightly above unity
    hmat =[alpha_2,alpha_1,alpha_0]; % Aim uses this order of alphas.
    
    [A,ErrFlagVal] = feedback(alphas,hmat,condn,uprbnd);
    
else
    
    [A] = findandcheckA(alphas,alpha_2,alpha_1,alpha_0);
    
end

betas=[beta_0,beta_1];

rho=[rhoz 0 0
      0 rhol 0
      0 0 rhox];
  
[B,P] = solveb(A,alphas,betas,rho,tau);

if max(max(tau==0)) > 0
    beta_0=[beta_0 zeros(2,3)];
    beta_1=[beta_1 zeros(2,3)];
end

F=alpha_0*A*B+alpha_0*B*P+alpha_1*B+beta_0*P+beta_1;

ttau=[1 0
    1 1
    1 0];

if max(max(abs(ttau-tau))) < .1e-10
    
    Ft=F;
    Ft(2,4)=F(2,4)+rhoz*F(2,1);
    Ft(2,6)=F(2,6)+rhox*F(2,3);
    Ft(2,1)=0;
    Ft(2,3)=0;
    if max(max(abs(Ft))) > .1e-10
        error('fatal (solution) feedforward part wrong')
    else
        disp('note(solution) feedforward passed ad hoc check, disregard warning from checksolve')
    end
    
else
    
    if max(max(abs(F))) > .1e-10
        error('fatal (solution) F failed accuracy check, something is wrong')
    end
    
end

%            
[a0,a1,a2,b0,b1] = rform(A,B,theta,delta,gamma,muz,k,c,y,l,def);

W=[sigz^2 0 0 
    0 sigl^2 0
    0 0 sigx^2];

if min(min(tau)) == 1
    V=W;
else
    V=zeros(6,6);
    V([1:3],[1:3])=W;
end