function [l,c,k,y,A,B,P,h0,h1] = solvemodel(theta,muz,taux,beta,delta,gamma,taul,psi,lb,rhoz,rhox,rhol,chk)

%This solves the model considered in the ckm paper on vars.
%The preferences are: 
%
%         sum(t=0....inf)(beta*(1+gamma))^t{log(C(t)) + psi*log(lb-l(t))}
%
%The household budget constraint is:
%
%   C(t) + (1+taux(t))*[(1+gamma)*K(t+1) - (1-delta)*K(t)] = (1-taul(t))*w(t)*l(t) + r(t)*K(t)
%
%The resource constraint is:
%
%   C(t) + (1+gamma)*K(t+1) - (1-delta)*K(t) = (K(t)^theta)*((Z(t)*l(t))^(1-theta))
%
%where the term to the right of the equality is the production function and Z(t) is a technology shock.
%The economy is competitive, so the rental rate on capital, r(t), and the
%wage rate, w(t), correspond to the relevant marginal productivities.
%Here, C(t), K(t), l(t) correspond to per capita consumption, capital and employment respectively.
%The gross growth rate of the population is 1+gamma.
%
%      The exogenous shocks obey the following relationship:
%
%   Z(t)/Z(t-1) = exp(muz+mu(t)), mu(t)=rhoz*mu(t-1) + sigz*epsz(t),
%   taul(t)-taul = rhol*(taul(t-1)-taul) + sigl*epsl(t)
%   taux(t)-taux = rhox*(taux(t-1)-taux) + sigx*epsx(t)
%
%The program scales C(t) and K(t+1) by Z(t), and calls these c(t) and k(t+1), respectively.
%The program returns the steady states, c, l, y, and k.
%It also returns the policy rules:
%
%  khat(t+1)=A*khat(t)+B*s(t)
%  lhat(t) = h0*khat(t) + h1*s(t),
%
%where xhat(t) denotes (x(t)-x)/x, and x is the steady state of x(t).
%Also, 
%
%      s(t) = (mu(t) (taul(t)-taul) (taux(t) - taux))', and 
%    eps(t) = (sigz*epsz(t) sigl*epsl(t) sigx*epsx(t))'
%
%      s(t)=P*s(t-1)+eps(t)
%
%if chk = 1, check all the analytic derivative formulas with their
%numerical counterparts.

%get the steady state:
if abs(sig-1) > .1e-10
    error('fatal (solvemodel) linearizeandsolve is not equipped to deal with sig .ne. 1.')
end
[y,l,c,k] = sstate(theta,muz,taux,beta,delta,gamma,taul,psi,lb,sig);

%get the linearized solution to the Euler equations:

[A,B,h0,h1,P] = linearizeandsolve(k,l,y,c,taul,taux,gamma,theta,delta,muz,psi,lb, ...
    beta,rhoz,rhox,rhol,chk);